using System;
using System.Windows;
using System.Windows.Media;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Xml.Serialization;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using NinjaTrader.NinjaScript;
using NinjaTrader.Core.FloatingPoint;
using NinjaTrader.NinjaScript.DrawingTools;
using System.Windows.Threading; 

namespace NinjaTrader.NinjaScript.Indicators
{
    public class TrueOpen : Indicator
    {
        #region Constants
        private const string TDO_LINE_ID = "TDOLine";
        private const string TSO_LINE_ID = "TSOLine";
        private const string TDO_TEXT_ID = "TDOText";
        private const string TSO_TEXT_ID = "TSOText";
        private const string PDH_LINE_ID = "PDHLine";
        private const string PDL_LINE_ID = "PDLLine";
        private const string PDH_TEXT_ID = "PDHText";
        private const string PDL_TEXT_ID = "PDLText";
        private const string PWH_LINE_ID = "PWHLine";
        private const string PWL_LINE_ID = "PWLLine";
        private const string PWH_TEXT_ID = "PWHText";
        private const string PWL_TEXT_ID = "PWLText";
        private const string LONDON_LINE_ID = "LondonLine";
        private const string NY_LINE_ID = "NYLine";
        #endregion

        #region Variables
        private DateTime lastTdoDate;
        private DateTime lastTsoDate;
        private double tdoPrice;
        private double tsoPrice;
        private NinjaTrader.Gui.Tools.SimpleFont textFont;
        private bool tdoInitialized;
        private bool tsoInitialized;
        private DateTime currentDay;
        private DateTime currentWeek;
        private double pdhPrice;
        private double pdlPrice;
        private double pwhPrice;
        private double pwlPrice;
        private DateTime londonSessionTime;
        private DateTime nySessionTime;
		private DateTime lastPdhTime;
		private DateTime lastPdlTime;
		private DateTime lastPwhTime;
		private DateTime lastPwlTime;
		private System.Windows.Threading.DispatcherTimer updateTimer;
		#endregion

        protected override void OnStateChange()
        {
            if (State == State.SetDefaults)
            {
                Description = "Displays True Day Open, Session Opens, and Previous High/Low levels";
                Name = "True Open";
                
                // Default parameter values
                GmtOffset = -6;
                BarsRight = 5;
                TdoColor = Brushes.DarkGray;
                TsoColor = Brushes.DarkGray;
                FontName = "Arial";
                FontSize = 12;
                
                // New parameters
                PdhColor = Brushes.DodgerBlue;
                PdlColor = Brushes.DodgerBlue;
                PdhStyle = DashStyleHelper.Solid;
                PdlStyle = DashStyleHelper.Solid;
                PdhWidth = 1;
                PdlWidth = 1;
                
                PwhColor = Brushes.Purple;
                PwlColor = Brushes.Purple;
                PwhStyle = DashStyleHelper.Dash;
                PwlStyle = DashStyleHelper.Dash;
                PwhWidth = 1;
                PwlWidth = 1;
                
                LondonColor = Brushes.Green;
                LondonStyle = DashStyleHelper.Dot;
                LondonWidth = 1;
                
                NyColor = Brushes.Orange;
                NyStyle = DashStyleHelper.Dot;
                NyWidth = 1;
                
                Calculate = Calculate.OnBarClose;
                IsOverlay = true;
                DisplayInDataBox = true;
                DrawOnPricePanel = true;
                IsAutoScale = true;
            }
            else if (State == State.Configure)
            {
                textFont = new NinjaTrader.Gui.Tools.SimpleFont(FontName, FontSize);
                currentDay = DateTime.MinValue;
                currentWeek = DateTime.MinValue;
            }
			else if (State == State.Realtime)
			{
				if (ChartControl != null)
	            {
	                ChartControl.Dispatcher.InvokeAsync(() =>
	                {
	                    if (updateTimer == null)
	                    {
//	                        Print("Creating timer..."); // Debug print
	                        updateTimer = new DispatcherTimer();
	                        updateTimer.Interval = TimeSpan.FromSeconds(10); // Reduced for testing
	                        updateTimer.Tick += OnUpdateTimer;
	                        updateTimer.Start();
//	                        Print("Timer started"); // Debug print
	                    }
	                });
	            }
			}
            else if (State == State.Terminated)
            {
				if (ChartControl != null && updateTimer != null)
	            {
	                ChartControl.Dispatcher.InvokeAsync(() =>
	                {
//	                    Print("Stopping timer..."); // Debug print
	                    updateTimer.Stop();
	                    updateTimer.Tick -= OnUpdateTimer;
	                    updateTimer = null;
//	                    Print("Timer stopped"); // Debug print
	                });
	            }
                ClearAllDrawObjects();
            }
        }
		
		private void OnUpdateTimer(object sender, EventArgs e)
		{
//		    Print($"Timer tick at {DateTime.Now}"); // Debug print
		    try 
		    {
		        if (ChartControl != null && ChartBars != null && CurrentBar >= 0)
		        {
//		            Print($"Current Time[0]: {Time[0]}"); // Debug time value
//		            Print($"State: {State}"); // Debug state
		            
		            DateTime now = Time[0];
		            DateTime todayLondon = now.Date.AddHours(londonSessionTime.Hour).AddMinutes(londonSessionTime.Minute);
		            DateTime todayNY = now.Date.AddHours(nySessionTime.Hour).AddMinutes(nySessionTime.Minute);
		            
//		            Print($"Now: {now}"); // Debug calculated times
//		            Print($"London Time: {todayLondon}");
//		            Print($"NY Time: {todayNY}");
		            
		            UpdateSessionLines();
//		            Print("UpdateSessionLines called"); // Debug print
		        }
		    }
		    catch (Exception ex)
		    {
		        Print($"Error in timer tick: {ex.Message}"); // Debug print
		        if (updateTimer != null)
		        {
		            updateTimer.Stop();
		            updateTimer = null;
		            Print("Timer stopped due to error"); // Debug print
		        }
		    }
		}
		
        private void ClearAllDrawObjects()
        {
            RemoveDrawObject(TDO_LINE_ID);
            RemoveDrawObject(TSO_LINE_ID);
            RemoveDrawObject(TDO_TEXT_ID);
            RemoveDrawObject(TSO_TEXT_ID);
            RemoveDrawObject(PDH_LINE_ID);
            RemoveDrawObject(PDL_LINE_ID);
            RemoveDrawObject(PDH_TEXT_ID);
            RemoveDrawObject(PDL_TEXT_ID);
            RemoveDrawObject(PWH_LINE_ID);
            RemoveDrawObject(PWL_LINE_ID);
            RemoveDrawObject(PWH_TEXT_ID);
            RemoveDrawObject(PWL_TEXT_ID);
            RemoveDrawObject(LONDON_LINE_ID);
            RemoveDrawObject(NY_LINE_ID);
        }

        protected override void OnBarUpdate()
        {
            if (CurrentBar < 1) return;

            try
            {
                DateTime currentTime = Time[0];
                DateTime adjustedDate = currentTime.Date;
                
                // Check for new day
                if (adjustedDate != currentDay)
                {
                    currentDay = adjustedDate;
                    InitializeNewDay(currentTime);
                }

                // Check for new week
                DateTime adjustedWeekStart = adjustedDate.AddDays(-(int)adjustedDate.DayOfWeek);
                if (adjustedWeekStart != currentWeek)
                {
                    currentWeek = adjustedWeekStart;
                    InitializeNewWeek();
                }

                // Update TDO/TSO
                UpdateTdoTso(currentTime);
                
                // Only update objects if initialized
                if (tdoInitialized)
                    UpdateTdoObjects();
                if (tsoInitialized)
                    UpdateTsoObjects();
                    
                UpdatePdhPdlObjects();
                UpdatePwhPwlObjects();
                UpdateSessionLines();
            }
            catch (Exception ex)
            {
                // Only log unexpected errors
                if (!ex.Message.Contains("barsAgo needed to be between"))
                {
//                    Print($"Unexpected error in OnBarUpdate: {ex.Message}");
                }
            }
        }

        private void InitializeNewDay(DateTime currentTime)
        {
            try
            {
                // Clear existing objects
                ClearAllDrawObjects();
                tdoInitialized = false;
                tsoInitialized = false;

                // Calculate previous day high/low
                double high = double.MinValue;
                double low = double.MaxValue;
                DateTime highTime = DateTime.MinValue;
                DateTime lowTime = DateTime.MinValue;
                
                // Get exact previous day date
                DateTime previousDate = currentTime.Date.AddDays(-1);
                
                // First find bars from previous day
                bool foundPreviousDay = false;
                for (int i = 1; i < 1000; i++)
                {
                    if (Time[i].Date == previousDate)
                    {
                        foundPreviousDay = true;
                        double currentHigh = High[i];
                        double currentLow = Low[i];
                        
                        if (currentHigh > high)
                        {
                            high = currentHigh;
                            highTime = Time[i];
                        }
                        if (currentLow < low)
                        {
                            low = currentLow;
                            lowTime = Time[i];
                        }
                    }
                    else if (Time[i].Date < previousDate)
                    {
                        break;
                    }
                }
                
                if (foundPreviousDay && highTime > DateTime.MinValue && lowTime > DateTime.MinValue)
                {
                    pdhPrice = high;
                    pdlPrice = low;
                    lastPdhTime = highTime;
                    lastPdlTime = lowTime;
//                    Print($"Found previous day prices - PDH: {high} at {highTime}, PDL: {low} at {lowTime}");
                }
                else
                {
//                    Print("No previous day data found");
                }

                // Set session times for the new day
                londonSessionTime = currentTime.Date.AddHours(3 - GmtOffset); // 3 AM
                nySessionTime = currentTime.Date.AddHours(9 - GmtOffset).AddMinutes(30); // 3:30 PM
            }
            catch (Exception ex)
            {
//                Print($"Error in InitializeNewDay: {ex.Message}");
            }
        }

        private void InitializeNewWeek()
        {
            // Calculate previous week high/low
            double high = double.MinValue;
            double low = double.MaxValue;
            DateTime highTime = DateTime.MinValue;
            DateTime lowTime = DateTime.MinValue;
            
            for (int i = 1; i < 1000; i++)
            {
                DateTime barDate = Time[i].AddHours(-GmtOffset).Date;
                if (barDate < currentWeek.AddDays(-7))
                    break;
                    
                if (High[i] > high)
                {
                    high = High[i];
                    highTime = Time[i];
                }
                if (Low[i] < low)
                {
                    low = Low[i];
                    lowTime = Time[i];
                }
            }
            
            if (high > double.MinValue)
            {
                pwhPrice = high;
                pwlPrice = low;
                lastPwhTime = highTime;
                lastPwlTime = lowTime;
            }
        }

        private void UpdateTdoTso(DateTime currentTime)
        {
            // Check and initialize TDO (Midnight EST)
            DateTime tdoTime = currentDay.AddHours(-GmtOffset);
            if (!tdoInitialized && currentTime >= tdoTime)
            {
                tdoPrice = Close[0];
                tdoInitialized = true;
                lastTdoDate = tdoTime;
            }

            // Check and initialize TSO (7:30 AM EST)
            DateTime tsoTime = currentDay.AddHours(-GmtOffset + 7).AddMinutes(30);
            if (!tsoInitialized && currentTime >= tsoTime)
            {
                tsoPrice = Close[0];
                tsoInitialized = true;
                lastTsoDate = tsoTime;
            }
        }

        private void UpdatePdhPdlObjects()
        {
            try 
            {
                if (lastPdhTime > DateTime.MinValue && lastPdlTime > DateTime.MinValue)
                {
                    // Calculate end time (3 bars into the future)
                    DateTime endTime = Time[0].AddSeconds((Time[0] - Time[1]).TotalSeconds * 3);

                    // Draw PDH line and text
                    Draw.Line(this, PDH_LINE_ID, false, lastPdhTime, pdhPrice, 
                        endTime, pdhPrice, PdhColor, PdhStyle, PdhWidth);
                    Draw.Text(this, PDH_TEXT_ID, false, "PDH", endTime, pdhPrice,
                        0, PdhColor, textFont, TextAlignment.Left, Brushes.Transparent,
                        Brushes.Transparent, 100);

                    // Draw PDL line and text
                    Draw.Line(this, PDL_LINE_ID, false, lastPdlTime, pdlPrice, 
                        endTime, pdlPrice, PdlColor, PdlStyle, PdlWidth);
                    Draw.Text(this, PDL_TEXT_ID, false, "PDL", endTime, pdlPrice,
                        0, PdlColor, textFont, TextAlignment.Left, Brushes.Transparent,
                        Brushes.Transparent, 100);
                }
            }
            catch (Exception ex)
            {
//                Print($"Error updating PDH/PDL objects: {ex.Message}");
            }
        }

        private void UpdatePwhPwlObjects()
        {
            // Only attempt to update if we have valid dates
            if (lastPwhTime > DateTime.MinValue && lastPwlTime > DateTime.MinValue)
            {
                try 
                {
                    // Calculate end time (3 bars into the future)
                    DateTime endTime = Time[0].AddSeconds((Time[0] - Time[1]).TotalSeconds * 3);

                    // Draw PWH line and text
                    Draw.Line(this, PWH_LINE_ID, false, lastPwhTime, pwhPrice, 
                        endTime, pwhPrice, PwhColor, PwhStyle, PwhWidth);
                    Draw.Text(this, PWH_TEXT_ID, false, "PWH", endTime, pwhPrice,
                        0, PwhColor, textFont, TextAlignment.Left, Brushes.Transparent,
                        Brushes.Transparent, 100);

                    // Draw PWL line and text
                    Draw.Line(this, PWL_LINE_ID, false, lastPwlTime, pwlPrice, 
                        endTime, pwlPrice, PwlColor, PwlStyle, PwlWidth);
                    Draw.Text(this, PWL_TEXT_ID, false, "PWL", endTime, pwlPrice,
                        0, PwlColor, textFont, TextAlignment.Left, Brushes.Transparent,
                        Brushes.Transparent, 100);
                }
                catch (Exception ex)
                {
                    // Only log truly unexpected errors
                    if (!ex.Message.Contains("startTime must be greater") && 
                        !ex.Message.Contains("barsAgo needed to be between"))
                    {
//                        Print($"Unexpected error updating PWH/PWL objects: {ex.Message}");
                    }
                }
            }
        }

        private void UpdateSessionLines()
		{
		    try
		    {
		        double range = ChartPanel.MaxValue - ChartPanel.MinValue;
		        double padding = range * 0.07;
		        double textY = ChartPanel.MaxValue - padding;
		
		        // Use actual current time instead of Time[0]
		        DateTime now = DateTime.Now;
		        DateTime todayLondon = now.Date.AddHours(londonSessionTime.Hour).AddMinutes(londonSessionTime.Minute);
		        DateTime todayNY = now.Date.AddHours(nySessionTime.Hour).AddMinutes(nySessionTime.Minute);
		
//		        Print($"UpdateSessionLines - Using times:"); // Debug prints
//		        Print($"  now: {now}");
//		        Print($"  todayLondon: {todayLondon}");
//		        Print($"  textY: {textY}");
		
		        // Get London bar index and convert to barsAgo
		        int londonBar = ChartBars.GetBarIdxByTime(ChartControl, todayLondon);
//		        Print($"London bar index: {londonBar}"); // Debug print
		        
		        if (londonBar >= 0)
		        {
		            int barsAgo = CurrentBar - londonBar;
		            
		            if (DrawObjects[LONDON_LINE_ID] == null)
		            {
		                Draw.VerticalLine(this, LONDON_LINE_ID, barsAgo, LondonColor, LondonStyle, LondonWidth);
//		                Print("Drew London line"); // Debug print
		            }
		            
		            var londonText = DrawObjects["LondonLabel"] as NinjaTrader.NinjaScript.DrawingTools.Text;
		            if (londonText != null)
		            {
		                foreach (ChartAnchor anchor in londonText.Anchors)
		                {
		                    anchor.Time = todayLondon;  
		                    anchor.Price = textY;
//		                    Print($"Updated London text anchor - Time: {anchor.Time}, Price: {anchor.Price}"); // Debug print
		                    break;
		                }
		            }
		            else
		            {
		                Draw.Text(this, "LondonLabel", true, "London", todayLondon, 
		                    textY, 0, LondonColor, textFont, TextAlignment.Center,
		                    Brushes.Transparent, Brushes.Transparent, 100);
//		                Print("Created new London text"); // Debug print
		            }
		        }
		
		        // NY Session handling
		        int nyBar = ChartBars.GetBarIdxByTime(ChartControl, todayNY);
//		        Print($"NY bar index: {nyBar}"); // Debug print
		        
		        if (nyBar >= 0)
		        {
		            int barsAgo = CurrentBar - nyBar;
		            
		            if (DrawObjects[NY_LINE_ID] == null)
		            {
		                Draw.VerticalLine(this, NY_LINE_ID, barsAgo, NyColor, NyStyle, NyWidth);
//		                Print("Drew NY line"); // Debug print
		            }
		            
		            var nyText = DrawObjects["NYLabel"] as NinjaTrader.NinjaScript.DrawingTools.Text;
		            if (nyText != null)
		            {
		                foreach (ChartAnchor anchor in nyText.Anchors)
		                {
		                    anchor.Time = todayNY;
		                    anchor.Price = textY;
//		                    Print($"Updated NY text anchor - Time: {anchor.Time}, Price: {anchor.Price}"); // Debug print
		                    break;
		                }
		            }
		            else
		            {
		                Draw.Text(this, "NYLabel", true, "NY", todayNY, 
		                    textY, 0, NyColor, textFont, TextAlignment.Center,
		                    Brushes.Transparent, Brushes.Transparent, 100);
//		                Print("Created new NY text"); // Debug print
		            }
		        }
		    }
		    catch (Exception ex)
		    {
		        Print($"Error in UpdateSessionLines: {ex.Message}"); // Debug print
		        if (ex.StackTrace != null)
		            Print($"Stack trace: {ex.StackTrace}"); // Debug stack trace
		    }
		}

		private void UpdateTdoObjects()
        {
            try
            {
                // Calculate end time (BarsRight bars into the future)
                DateTime endTime = Time[0].AddSeconds(
                    (Time[0] - Time[1]).TotalSeconds * BarsRight);

                // Update or create the TDO line
                Draw.Line(this, TDO_LINE_ID, false, lastTdoDate, tdoPrice, 
                    endTime, tdoPrice, TdoColor, DashStyleHelper.Solid, 1);

                // Update or create the TDO text
                DateTime textTime = endTime.AddSeconds(
                    (Time[0] - Time[1]).TotalSeconds); // One bar offset
                Draw.Text(this, TDO_TEXT_ID, false, "Midnight", textTime, 
                    tdoPrice, 0, TdoColor, textFont, TextAlignment.Left, 
                    Brushes.Transparent, Brushes.Transparent, 100);
            }
            catch (Exception ex)
            {
//                Print($"Error updating TDO objects: {ex.Message}");
            }
        }

        private void UpdateTsoObjects()
        {
            try
            {
                // Calculate end time (BarsRight bars into the future)
                DateTime endTime = Time[0].AddSeconds(
                    (Time[0] - Time[1]).TotalSeconds * BarsRight);

                // Update or create the TSO line
                Draw.Line(this, TSO_LINE_ID, false, lastTsoDate, tsoPrice,
                    endTime, tsoPrice, TsoColor, DashStyleHelper.Solid, 1);

                // Update or create the TSO text
                DateTime textTime = endTime.AddSeconds(
                    (Time[0] - Time[1]).TotalSeconds); // One bar offset
                Draw.Text(this, TSO_TEXT_ID, false, "7:30 AM", textTime,
                    tsoPrice, 0, TsoColor, textFont, TextAlignment.Left,
                    Brushes.Transparent, Brushes.Transparent, 100);
            }
            catch (Exception ex)
            {
//                Print($"Error updating TSO objects: {ex.Message}");
            }
        }
		
		#region Original Properties
        [NinjaScriptProperty]
        [PropertyEditor("NinjaTrader.Gui.Tools.RangeEditor")]
        [Range(-12, 12)]
        [Display(Name = "GMT Offset",
                Description = "GMT offset for time calculations",
                Order = 1,
                GroupName = "Parameters")]
        public int GmtOffset { get; set; }

        [NinjaScriptProperty]
        [PropertyEditor("NinjaTrader.Gui.Tools.RangeEditor")]
        [Range(1, 50)]
        [Display(Name = "Bars Right",
                Description = "Number of bars to extend lines to the right",
                Order = 2,
                GroupName = "Parameters")]
        public int BarsRight { get; set; }

        [NinjaScriptProperty]
        [XmlIgnore]
        [Display(Name = "TDO Color",
                Description = "Color for True Day Open line",
                Order = 3,
                GroupName = "Parameters")]
        public Brush TdoColor { get; set; }

        [Browsable(false)]
        public string TdoColorSerializable
        {
            get { return Serialize.BrushToString(TdoColor); }
            set { TdoColor = Serialize.StringToBrush(value); }
        }

        [NinjaScriptProperty]
        [XmlIgnore]
        [Display(Name = "TSO Color",
                Description = "Color for True Session Open line",
                Order = 4,
                GroupName = "Parameters")]
        public Brush TsoColor { get; set; }

        [Browsable(false)]
        public string TsoColorSerializable
        {
            get { return Serialize.BrushToString(TsoColor); }
            set { TsoColor = Serialize.StringToBrush(value); }
        }

        [NinjaScriptProperty]
        [Display(Name = "Font Name",
                Description = "Font name for text labels",
                Order = 5,
                GroupName = "Parameters")]
        public string FontName { get; set; }

        [NinjaScriptProperty]
        [PropertyEditor("NinjaTrader.Gui.Tools.RangeEditor")]
        [Range(8, 48)]
        [Display(Name = "Font Size",
                Description = "Font size for text labels",
                Order = 6,
                GroupName = "Parameters")]
        public int FontSize { get; set; }
        #endregion
		
		#region Properties
        // Original properties remain unchanged...

        [NinjaScriptProperty]
        [XmlIgnore]
        [Display(Name = "PDH Color",
                Description = "Color for Previous Day High line",
                Order = 7,
                GroupName = "Previous Day")]
        public Brush PdhColor { get; set; }

        [Browsable(false)]
        public string PdhColorSerializable
        {
            get { return Serialize.BrushToString(PdhColor); }
            set { PdhColor = Serialize.StringToBrush(value); }
        }

        [NinjaScriptProperty]
        [Display(Name = "PDH Style",
                Description = "Style for Previous Day High line",
                Order = 8,
                GroupName = "Previous Day")]
        public DashStyleHelper PdhStyle { get; set; }

        [NinjaScriptProperty]
        [Range(1, 10)]
        [Display(Name = "PDH Width",
                Description = "Width for Previous Day High line",
                Order = 9,
                GroupName = "Previous Day")]
        public int PdhWidth { get; set; }

        [NinjaScriptProperty]
        [XmlIgnore]
        [Display(Name = "PDL Color",
                Description = "Color for Previous Day Low line",
                Order = 10,
                GroupName = "Previous Day")]
        public Brush PdlColor { get; set; }

        [Browsable(false)]
        public string PdlColorSerializable
        {
            get { return Serialize.BrushToString(PdlColor); }
            set { PdlColor = Serialize.StringToBrush(value); }
        }

        [NinjaScriptProperty]
        [Display(Name = "PDL Style",
                Description = "Style for Previous Day Low line",
                Order = 11,
                GroupName = "Previous Day")]
        public DashStyleHelper PdlStyle { get; set; }

        [NinjaScriptProperty]
        [Range(1, 10)]
        [Display(Name = "PDL Width",
                Description = "Width for Previous Day Low line",
                Order = 12,
                GroupName = "Previous Day")]
        public int PdlWidth { get; set; }

        [NinjaScriptProperty]
        [XmlIgnore]
        [Display(Name = "PWH Color",
                Description = "Color for Previous Week High line",
                Order = 13,
                GroupName = "Previous Week")]
        public Brush PwhColor { get; set; }

        [Browsable(false)]
        public string PwhColorSerializable
        {
            get { return Serialize.BrushToString(PwhColor); }
            set { PwhColor = Serialize.StringToBrush(value); }
        }

        [NinjaScriptProperty]
        [Display(Name = "PWH Style",
                Description = "Style for Previous Week High line",
                Order = 14,
                GroupName = "Previous Week")]
        public DashStyleHelper PwhStyle { get; set; }


		[NinjaScriptProperty]
        [Range(1, 10)]
        [Display(Name = "PWH Width",
                Description = "Width for Previous Week High line",
                Order = 15,
                GroupName = "Previous Week")]
        public int PwhWidth { get; set; }

        [NinjaScriptProperty]
        [XmlIgnore]
        [Display(Name = "PWL Color",
                Description = "Color for Previous Week Low line",
                Order = 16,
                GroupName = "Previous Week")]
        public Brush PwlColor { get; set; }

        [Browsable(false)]
        public string PwlColorSerializable
        {
            get { return Serialize.BrushToString(PwlColor); }
            set { PwlColor = Serialize.StringToBrush(value); }
        }

        [NinjaScriptProperty]
        [Display(Name = "PWL Style",
                Description = "Style for Previous Week Low line",
                Order = 17,
                GroupName = "Previous Week")]
        public DashStyleHelper PwlStyle { get; set; }

        [NinjaScriptProperty]
        [Range(1, 10)]
        [Display(Name = "PWL Width",
                Description = "Width for Previous Week Low line",
                Order = 18,
                GroupName = "Previous Week")]
        public int PwlWidth { get; set; }

        [NinjaScriptProperty]
        [XmlIgnore]
        [Display(Name = "London Session Color",
                Description = "Color for London Session start line",
                Order = 19,
                GroupName = "Sessions")]
        public Brush LondonColor { get; set; }

        [Browsable(false)]
        public string LondonColorSerializable
        {
            get { return Serialize.BrushToString(LondonColor); }
            set { LondonColor = Serialize.StringToBrush(value); }
        }

        [NinjaScriptProperty]
        [Display(Name = "London Session Style",
                Description = "Style for London Session start line",
                Order = 20,
                GroupName = "Sessions")]
        public DashStyleHelper LondonStyle { get; set; }

        [NinjaScriptProperty]
        [Range(1, 10)]
        [Display(Name = "London Session Width",
                Description = "Width for London Session start line",
                Order = 21,
                GroupName = "Sessions")]
        public int LondonWidth { get; set; }

        [NinjaScriptProperty]
        [XmlIgnore]
        [Display(Name = "NY Session Color",
                Description = "Color for NY Session start line",
                Order = 22,
                GroupName = "Sessions")]
        public Brush NyColor { get; set; }

        [Browsable(false)]
        public string NyColorSerializable
        {
            get { return Serialize.BrushToString(NyColor); }
            set { NyColor = Serialize.StringToBrush(value); }
        }

        [NinjaScriptProperty]
        [Display(Name = "NY Session Style",
                Description = "Style for NY Session start line",
                Order = 23,
                GroupName = "Sessions")]
        public DashStyleHelper NyStyle { get; set; }

        [NinjaScriptProperty]
        [Range(1, 10)]
        [Display(Name = "NY Session Width",
                Description = "Width for NY Session start line",
                Order = 24,
                GroupName = "Sessions")]
        public int NyWidth { get; set; }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.

namespace NinjaTrader.NinjaScript.Indicators
{
	public partial class Indicator : NinjaTrader.Gui.NinjaScript.IndicatorRenderBase
	{
		private TrueOpen[] cacheTrueOpen;
		public TrueOpen TrueOpen(int gmtOffset, int barsRight, Brush tdoColor, Brush tsoColor, string fontName, int fontSize, Brush pdhColor, DashStyleHelper pdhStyle, int pdhWidth, Brush pdlColor, DashStyleHelper pdlStyle, int pdlWidth, Brush pwhColor, DashStyleHelper pwhStyle, int pwhWidth, Brush pwlColor, DashStyleHelper pwlStyle, int pwlWidth, Brush londonColor, DashStyleHelper londonStyle, int londonWidth, Brush nyColor, DashStyleHelper nyStyle, int nyWidth)
		{
			return TrueOpen(Input, gmtOffset, barsRight, tdoColor, tsoColor, fontName, fontSize, pdhColor, pdhStyle, pdhWidth, pdlColor, pdlStyle, pdlWidth, pwhColor, pwhStyle, pwhWidth, pwlColor, pwlStyle, pwlWidth, londonColor, londonStyle, londonWidth, nyColor, nyStyle, nyWidth);
		}

		public TrueOpen TrueOpen(ISeries<double> input, int gmtOffset, int barsRight, Brush tdoColor, Brush tsoColor, string fontName, int fontSize, Brush pdhColor, DashStyleHelper pdhStyle, int pdhWidth, Brush pdlColor, DashStyleHelper pdlStyle, int pdlWidth, Brush pwhColor, DashStyleHelper pwhStyle, int pwhWidth, Brush pwlColor, DashStyleHelper pwlStyle, int pwlWidth, Brush londonColor, DashStyleHelper londonStyle, int londonWidth, Brush nyColor, DashStyleHelper nyStyle, int nyWidth)
		{
			if (cacheTrueOpen != null)
				for (int idx = 0; idx < cacheTrueOpen.Length; idx++)
					if (cacheTrueOpen[idx] != null && cacheTrueOpen[idx].GmtOffset == gmtOffset && cacheTrueOpen[idx].BarsRight == barsRight && cacheTrueOpen[idx].TdoColor == tdoColor && cacheTrueOpen[idx].TsoColor == tsoColor && cacheTrueOpen[idx].FontName == fontName && cacheTrueOpen[idx].FontSize == fontSize && cacheTrueOpen[idx].PdhColor == pdhColor && cacheTrueOpen[idx].PdhStyle == pdhStyle && cacheTrueOpen[idx].PdhWidth == pdhWidth && cacheTrueOpen[idx].PdlColor == pdlColor && cacheTrueOpen[idx].PdlStyle == pdlStyle && cacheTrueOpen[idx].PdlWidth == pdlWidth && cacheTrueOpen[idx].PwhColor == pwhColor && cacheTrueOpen[idx].PwhStyle == pwhStyle && cacheTrueOpen[idx].PwhWidth == pwhWidth && cacheTrueOpen[idx].PwlColor == pwlColor && cacheTrueOpen[idx].PwlStyle == pwlStyle && cacheTrueOpen[idx].PwlWidth == pwlWidth && cacheTrueOpen[idx].LondonColor == londonColor && cacheTrueOpen[idx].LondonStyle == londonStyle && cacheTrueOpen[idx].LondonWidth == londonWidth && cacheTrueOpen[idx].NyColor == nyColor && cacheTrueOpen[idx].NyStyle == nyStyle && cacheTrueOpen[idx].NyWidth == nyWidth && cacheTrueOpen[idx].EqualsInput(input))
						return cacheTrueOpen[idx];
			return CacheIndicator<TrueOpen>(new TrueOpen(){ GmtOffset = gmtOffset, BarsRight = barsRight, TdoColor = tdoColor, TsoColor = tsoColor, FontName = fontName, FontSize = fontSize, PdhColor = pdhColor, PdhStyle = pdhStyle, PdhWidth = pdhWidth, PdlColor = pdlColor, PdlStyle = pdlStyle, PdlWidth = pdlWidth, PwhColor = pwhColor, PwhStyle = pwhStyle, PwhWidth = pwhWidth, PwlColor = pwlColor, PwlStyle = pwlStyle, PwlWidth = pwlWidth, LondonColor = londonColor, LondonStyle = londonStyle, LondonWidth = londonWidth, NyColor = nyColor, NyStyle = nyStyle, NyWidth = nyWidth }, input, ref cacheTrueOpen);
		}
	}
}

namespace NinjaTrader.NinjaScript.MarketAnalyzerColumns
{
	public partial class MarketAnalyzerColumn : MarketAnalyzerColumnBase
	{
		public Indicators.TrueOpen TrueOpen(int gmtOffset, int barsRight, Brush tdoColor, Brush tsoColor, string fontName, int fontSize, Brush pdhColor, DashStyleHelper pdhStyle, int pdhWidth, Brush pdlColor, DashStyleHelper pdlStyle, int pdlWidth, Brush pwhColor, DashStyleHelper pwhStyle, int pwhWidth, Brush pwlColor, DashStyleHelper pwlStyle, int pwlWidth, Brush londonColor, DashStyleHelper londonStyle, int londonWidth, Brush nyColor, DashStyleHelper nyStyle, int nyWidth)
		{
			return indicator.TrueOpen(Input, gmtOffset, barsRight, tdoColor, tsoColor, fontName, fontSize, pdhColor, pdhStyle, pdhWidth, pdlColor, pdlStyle, pdlWidth, pwhColor, pwhStyle, pwhWidth, pwlColor, pwlStyle, pwlWidth, londonColor, londonStyle, londonWidth, nyColor, nyStyle, nyWidth);
		}

		public Indicators.TrueOpen TrueOpen(ISeries<double> input , int gmtOffset, int barsRight, Brush tdoColor, Brush tsoColor, string fontName, int fontSize, Brush pdhColor, DashStyleHelper pdhStyle, int pdhWidth, Brush pdlColor, DashStyleHelper pdlStyle, int pdlWidth, Brush pwhColor, DashStyleHelper pwhStyle, int pwhWidth, Brush pwlColor, DashStyleHelper pwlStyle, int pwlWidth, Brush londonColor, DashStyleHelper londonStyle, int londonWidth, Brush nyColor, DashStyleHelper nyStyle, int nyWidth)
		{
			return indicator.TrueOpen(input, gmtOffset, barsRight, tdoColor, tsoColor, fontName, fontSize, pdhColor, pdhStyle, pdhWidth, pdlColor, pdlStyle, pdlWidth, pwhColor, pwhStyle, pwhWidth, pwlColor, pwlStyle, pwlWidth, londonColor, londonStyle, londonWidth, nyColor, nyStyle, nyWidth);
		}
	}
}

namespace NinjaTrader.NinjaScript.Strategies
{
	public partial class Strategy : NinjaTrader.Gui.NinjaScript.StrategyRenderBase
	{
		public Indicators.TrueOpen TrueOpen(int gmtOffset, int barsRight, Brush tdoColor, Brush tsoColor, string fontName, int fontSize, Brush pdhColor, DashStyleHelper pdhStyle, int pdhWidth, Brush pdlColor, DashStyleHelper pdlStyle, int pdlWidth, Brush pwhColor, DashStyleHelper pwhStyle, int pwhWidth, Brush pwlColor, DashStyleHelper pwlStyle, int pwlWidth, Brush londonColor, DashStyleHelper londonStyle, int londonWidth, Brush nyColor, DashStyleHelper nyStyle, int nyWidth)
		{
			return indicator.TrueOpen(Input, gmtOffset, barsRight, tdoColor, tsoColor, fontName, fontSize, pdhColor, pdhStyle, pdhWidth, pdlColor, pdlStyle, pdlWidth, pwhColor, pwhStyle, pwhWidth, pwlColor, pwlStyle, pwlWidth, londonColor, londonStyle, londonWidth, nyColor, nyStyle, nyWidth);
		}

		public Indicators.TrueOpen TrueOpen(ISeries<double> input , int gmtOffset, int barsRight, Brush tdoColor, Brush tsoColor, string fontName, int fontSize, Brush pdhColor, DashStyleHelper pdhStyle, int pdhWidth, Brush pdlColor, DashStyleHelper pdlStyle, int pdlWidth, Brush pwhColor, DashStyleHelper pwhStyle, int pwhWidth, Brush pwlColor, DashStyleHelper pwlStyle, int pwlWidth, Brush londonColor, DashStyleHelper londonStyle, int londonWidth, Brush nyColor, DashStyleHelper nyStyle, int nyWidth)
		{
			return indicator.TrueOpen(input, gmtOffset, barsRight, tdoColor, tsoColor, fontName, fontSize, pdhColor, pdhStyle, pdhWidth, pdlColor, pdlStyle, pdlWidth, pwhColor, pwhStyle, pwhWidth, pwlColor, pwlStyle, pwlWidth, londonColor, londonStyle, londonWidth, nyColor, nyStyle, nyWidth);
		}
	}
}

#endregion
